% Parson bridge modal id - experimental
% Time-stamp: <2022-06-13 16:48:03 andre>
% General data [kg,m,s]
close all, clear all % Clean everything             
rng default;         % For repeatability

set(groot, 'defaultAxesTickLabelInterpreter','latex'); % For LaTeX
set(groot, 'defaultLegendInterpreter','latex');        % For LaTeX
set(0,'DefaultTextInterpreter','latex');               % For LaTeX
set(0,'DefaultAxesFontSize', 16),                      % Font Size

% Reference modal properties values
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
load('Modeshapes.mat'); phi = cell2mat(PMS(2:end,5:end)); phi = phi([2,5],[2,4,7]); % load mode shapes
load('Parson_amp_dep_mp_intervals.mat'), nsp2 = nsp;
Psn = struct('nf',[4.7,15.1,28.9],...    % natural frequencies
             'phi',phi,...               % mode shapes
             'dr',1e-2*[2.3,2.8,2.7],... % damping ratio 
             'mm',[862,907,839]);        % modal mass

% Prior pdf of Calibration parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% uniform
pr.nf = [4.78,14.42,29.27];    % natural frequencies (walking test)
pr.dr = 1e-2*[2.53,2.53,2.53]; % damping ratios (free decay)
pr.mm  = [862,907,839];        % modal masses (from FE)
pr.Se = [2e-4];                % PSD of noise (m^2/s^4/Hz)
pr.L =[0.0000;...
       3.3800;...
       5.0700;...
       6.7600;...
       8.8700;...
       10.9800;...
       12.6700;...
       14.3600;...
       16.9000];
         
tcp = [Psn.nf,Psn.dr,Psn.mm,NaN(1)];      % true calibration parameters
tcp(4:6)=100*tcp(4:6);
%%%% Observations Ye
%%%%%%%%%%%%%%%%%%%%%
td={'W201','W202','W203','W204','W205','W206','W207','W208','W209','W210','W211','W212'}; % m/s2 256 Hz
sp={'1p40','1p55','1p60','1p75','1p80','1p88','2p00','2p05','2p20','2p25','2p30','2p45'};        % speed
nsp=[1.40,1.55,1.60,1.75,1.80,1.88,2.00,2.05,2.20,2.25,2.30,2.45];                             % speed

amp = zeros(2,12,12); cdof = [2,5];
for rp = 1:2
    for ind = 1:length(sp)
        Ye = [];
        file=['SZ_Mea_',sp{ind},'_',int2str(rp),'new.mat'], load(file)
        Ye = [TP2new',TP5new'];
        lout = struct('L',pr.L,'cdof',cdof,'nr',rp,'sf',0.1,'tr',2,'cf',32); 

        % Load human walking forces data %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        load([td{ind},'n.mat']) % Load treadmill data
        F = Vdata(:,3);

        [lsm] = nspba(Ye,1/(tnew(2)-tnew(1)),F,1/dt,pr,lout); 
        % PLOTS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        lsm(4:6)=100*lsm(4:6); tcp(10:12)=ones(1,3);
        ephi = lsm(10:end-1); ephi = reshape(ephi,size(Ye,2),3);
        % store mp
        amp(rp,1:9,ind) = amp(rp,1:9,ind)+lsm(1:9);
        % find MACs
        load('Exp_modes_Parsons','V*');
        V1 = (V1(:,3)+V1(:,4))/2; V2=(V2(:,3)+V2(:,4))/2; V3=(V3(:,3)+V3(:,4))/2;
        V1c = V1; V2c = V2; V3c = V3;
        V1 = V1(cdof); V2 = V2(cdof); V3 = V3(cdof);
        for i = 1:3
            figure(9+i)
            eval(['mac1=transpose(V',int2str(i),')*ephi(:,i)/(norm(V',int2str(i),')*norm(ephi(:,i)));'])
            iephi=-1*ephi(:,i);
            eval(['mac2=transpose(V',int2str(i),')*iephi/(norm(V',int2str(i),')*norm(iephi));'])
            mac = max(mac1,mac2);
            % store MAC
            amp(rp,9+i,ind) = amp(rp,9+i,ind) + mac; lsm(9+i) = mac;
            if mac > 0.75
                if mac2 > 0.75, ephi(:,i)=-1*ephi(:,i); end
                eval(['h1=plot(pr.L(cdof),ephi(:,i),"-or","MarkerSize",10); hold on']);
            end
            grid on, xticks(pr.L'), xticklabels({'TP1','TP2','TP3','TP4','TP5','TP6','TP7','TP8','TP9',}), xlim([pr.L(1) pr.L(end)])
            if ind == 12 && rp == 2
                eval(['h2=plot(pr.L,V',int2str(i),'c,"-kx","MarkerSize",10);']);
                legend([h1,h2],{'Predicted','Reference'})
            end
        end
        for mp = 1:9
            figure(mp)
            switch rp
              case 1
                plot(nsp(ind),lsm(mp),'-sg','MarkerSize',10), hold on
              case 2
                plot(nsp(ind),lsm(mp),'-xb','MarkerSize',10), hold on            
              case 3
                plot(nsp(ind),lsm(mp),'-ok','MarkerSize',10), hold on            
            end   
            if ind == 1 && rp == 1
                switch mp
                  case 1
                    fill([nsp,fliplr(nsp)],[minfn,fliplr(maxfn)],'red','FaceAlpha',.3)         % Confidence int
                  case 4
                    fill([nsp,fliplr(nsp)],100*[minzeta,fliplr(maxzeta)],'red','FaceAlpha',.3) % Confidence int
                  otherwise
                    plot([nsp(1),nsp(end)],[tcp(mp),tcp(mp)],'r--')    
                end
                switch true
                  case mp<=3
                    xlabel('Pacing rate (Hz)')        
                    ylabel(['$f_',int2str(mp),'$ (Hz)'])
                  case mp<=6
                    xlabel('Pacing rate (Hz)')
                    ylabel(['$\zeta_',int2str(mp-3),'$ (\%)'])
                  case mp<=9
                    xlabel('Pacing rate (Hz)')
                    ylabel(['$m_',int2str(mp-6),'$ (kg)'])
                end
            end
        end % mp
    end
end
tcp(4:6)=100*tcp(4:6);
ferr=(mean(amp,3,'omitnan')-tcp)./tcp*100
[ferr(:,1:3),ferr(:,10:12);
 ferr(:,4:9)]
