% Parson bridge numerical example - Pedestrian-based approach 
% Time-stamp: <2022-06-14 20:06:16 andre>
% General data [kg,m,s]
close all, clear all % Clean everything
rng default;         % For repeatability

% Prior pdf of Calibration parameters = initial point of optimisation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
pr.nf = [4.9,15.8,31.4];       % natural frequencies (Hz)
pr.dr = [2.25,2.25,2.25]*1e-2; % damping ratios (%)
pr.mm  = [862,907,839];        % modal masses (kg)
pr.Se = [2e-4];                % PSD of noise (m^2/s^4/Hz)

L =[0.0000;...
    3.3800;...
    5.0700;...
    6.7600;...
    8.8700;...
    10.9800;...
    12.6700;...
    14.3600;...
    16.9000];

%%%% Observations Ye
%%%%%%%%%%%%%%%%%%%%%
nsp=[1.4,1.55,1.6,1.75,1.8,1.88,2.0,2.05,2.2,2.25,2.3,2.45];
load Parson_amp_dep_mp_intervals.mat
cdof = [2:8]; 
% Load bridge response data %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
file=['Resp1_to_force_W201_at_1p4Hz.mat']
load(file); Ye = [TP1a',TP2a',TP3a',TP4a',TP5a',TP6a',TP7a',TP8a',TP9a'];
Ye(end,:)=[]; Ye = Ye(:,cdof); Fs = 1/(t(2)-t(1));
% Add noise, SNR of 20db
snrdb = 20.*ones(1,length(cdof)); sig2 = mean(Ye.^2,1)./exp(snrdb/10);
Ye = Ye + repmat(sqrt(sig2),size(Ye,1),1).*randn(size(Ye));
% true calibration parameters                      
tcp = [fnV',zetaV',mV',NaN(1,1)]; tcp(4:6)=100*tcp(4:6);   
extra = struct('L',L,'cdof',cdof,'tr',2,'nr',1,'sf',0.1,'cf',32);

% Load human walking forces data %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
load(['W201n.mat']) % Load treadmill data
F = Vdata(:,3);

% Call main function %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
oef=1; [lsm,oef] = nspba(Ye,Fs,F,1/dt,pr,extra);

% PLOTS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
lsm(4:6)=100*lsm(4:6); tcp(10:12)=ones(1,3);
ephi = lsm(10:end-1); ephi = reshape(ephi,size(Ye,2),3);
% find MACs
load('Exp_modes_Parsons','V*');
V1 = (V1(:,3)+V1(:,4))/2; V2=(V2(:,3)+V2(:,4))/2; V3=(V3(:,3)+V3(:,4))/2;
V1c = V1; V2c = V2; V3c = V3;
V1 = V1(cdof); V2 = V2(cdof); V3 = V3(cdof);
for i = 1:3
    figure(12+i)
    % Compute mac
    eval(['mac1=transpose(V',int2str(i),')*ephi(:,i)/(norm(V',int2str(i),')*norm(ephi(:,i)));'])
    iephi=-1*ephi(:,i);         % invert mode shape
    eval(['mac2=transpose(V',int2str(i),')*iephi/(norm(V',int2str(i),')*norm(iephi));'])
    mac = max(mac1,mac2);
    if  mac > 0.75
        if mac2 > 0.75, ephi(:,i)=-1*ephi(:,i); end
        eval(['h1=plot(L(cdof),ephi(:,i),"-or","MarkerSize",10); hold on']);
    end 
    grid on, xticks(L'), xticklabels({'TP1','TP2','TP3','TP4','TP5','TP6','TP7','TP8','TP9',}), xlim([L(1) L(end)])
    eval(['h2=plot(L,V',int2str(i),'c,"-kx","MarkerSize",10);']);
    legend([h1,h2],{'Predicted','True'})
end
for mp = 1:12
    figure(mp)
    plot(1.4,lsm(mp),'-sg','MarkerSize',10), hold on
    % true values
    switch mp
      case 1
        fill([nsp,fliplr(nsp)],[minfn,fliplr(maxfn)],'red','FaceAlpha',.3)         % Confidence int
      case 4
        fill([nsp,fliplr(nsp)],100*[minzeta,fliplr(maxzeta)],'red','FaceAlpha',.3) % Confidence int
      otherwise
        plot([nsp(1),nsp(end)],[tcp(mp),tcp(mp)],'r--')    
    end
    switch true
      case mp<=3
        xlabel('Pacing rate (Hz)')        
        ylabel(['f_',int2str(mp),' (Hz)'])
      case mp<=6
        xlabel('Pacing rate (Hz)')
        ylabel(['\zeta_',int2str(mp-3),'(%)'])
      case mp<=9
        xlabel('Pacing rate (Hz)')
        ylabel(['m_',int2str(mp-6),' (kg)'])
      case mp<=12
        xlabel('Pacing rate (Hz)')
        ylabel(['MAC ',int2str(mp-9)])
    end
    vline(tcp(1)/2,'--k'), vline(tcp(1)/3,'-.k')
end