% Parson bridge numerical example - Pedestrian-based approach 
% Time-stamp: <2022-06-13 16:50:30 andre>
% General data [kg,m,s]
close all, clear all % Clean everything
rng default;         % For repeatability

set(groot,'defaultAxesTickLabelInterpreter','latex'); % For LaTeX
set(groot,'defaultLegendInterpreter','latex');        % For LaTeX
set(0,'DefaultTextInterpreter','latex');              % For LaTeX
set(0,'DefaultAxesFontSize', 16)                      % Font Size

% Prior pdf of Calibration parameters = initial point of optimisation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
pr.nf = [4.9,15.8,31.4];       % natural frequencies (Hz)
pr.dr = [2.25,2.25,2.25]*1e-2; % damping ratios (%)
pr.mm  = [862,907,839];        % modal masses (kg)
pr.Se = [2e-4];                % PSD of noise (m^2/s^4/Hz)

L =[0.0000;...
    3.3800;...
    5.0700;...
    6.7600;...
    8.8700;...
    10.9800;...
    12.6700;...
    14.3600;...
    16.9000];

%%%% Observations Ye
%%%%%%%%%%%%%%%%%%%%%
td={'W201','W202','W203','W204','W205','W206','W207','W208','W209','W210','W211','W212'}; % m/s2 256 Hz
sp={'1p4Hz','1p55Hz','1p6Hz','1p75Hz','1p8Hz','1p88Hz','2p0Hz','2p05Hz','2p2Hz','2p25Hz','2p3Hz','2p45Hz'}; % speed
nsp=[1.4,1.55,1.6,1.75,1.8,1.88,2.0,2.05,2.2,2.25,2.3,2.45];
load Parson_amp_dep_mp_intervals.mat
nm=length(pr.nf);                       % number of modes
nw=length(td);                          % number of walks
nmp=nm*4;                               % number of modal properties
nr=3;                                   % number of runs

amp = zeros(nr,nmp,nw); cdof = [2,3,5,7]; 
for rp = 1:nr
    for ind = 1:nw
        % Load bridge response data %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        Ye = [];
        file=['Resp',int2str(rp),'_to_force_',td{ind},'_at_',sp{ind},'.mat']
        load(file); Ye = [TP1a',TP2a',TP3a',TP4a',TP5a',TP6a',TP7a',TP8a',TP9a'];
        Ye(end,:)=[]; Ye = Ye(:,cdof); Fs = 1/(t(2)-t(1));
        % Add noise, SNR of 20db
        snrdb = 20.*ones(1,length(cdof)); 
        sig2 = mean(Ye.^2,1)./exp(snrdb/10);
        Ye = Ye + repmat(sqrt(sig2),size(Ye,1),1).*randn(size(Ye));
        % true calibration parameters                      
        tcp = [fnV',zetaV',mV',NaN(1,1)]; tcp(4:6)=100*tcp(4:6);   
        extra = struct('L',L,'cdof',cdof,'tr',2,'nr',rp,'sf',0.1,'cf',32);

        % Load human walking forces data %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        load([td{ind},'n.mat']) % Load treadmill data
        F = Vdata(:,3);

        % Call main function %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        oef=1; [lsm,oef] = nspba(Ye,Fs,F,1/dt,pr,extra);

        % PLOTS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        lsm(4:6)=100*lsm(4:6); tcp(10:12)=ones(1,3);
        ephi = lsm(10:end-1); ephi = reshape(ephi,size(Ye,2),3);
        % store mp
        if oef <= 0
            amp(rp,1:9,ind) = amp(rp,1:9,ind)+NaN(1,9);           
        else
            amp(rp,1:9,ind) = amp(rp,1:9,ind)+lsm(1:9);           
        end
        % find MACs
        load('Exp_modes_Parsons','V*');
        V1 = (V1(:,3)+V1(:,4))/2; V2=(V2(:,3)+V2(:,4))/2; V3=(V3(:,3)+V3(:,4))/2;
        V1c = V1; V2c = V2; V3c = V3;
        V1 = V1(cdof); V2 = V2(cdof); V3 = V3(cdof);
        for i = 1:3
            figure(12+i)
            % Compute mac
            eval(['mac1=transpose(V',int2str(i),')*ephi(:,i)/(norm(V',int2str(i),')*norm(ephi(:,i)));'])
            iephi=-1*ephi(:,i);         % invert mode shape
            eval(['mac2=transpose(V',int2str(i),')*iephi/(norm(V',int2str(i),')*norm(iephi));'])
            mac = max(mac1,mac2);
            % store MAC
            if oef <= 0
                amp(rp,9+i,ind) = amp(rp,9+i,ind) + NaN; lsm(9+i) = mac;
            else
                amp(rp,9+i,ind) = amp(rp,9+i,ind) + mac; lsm(9+i) = mac;
            end
            if  mac > 0.75
                if mac2 > 0.75, ephi(:,i)=-1*ephi(:,i); end
                eval(['h1=plot(L(cdof),ephi(:,i),"-or","MarkerSize",10); hold on']);
            end 
            grid on, xticks(L'), xticklabels({'TP1','TP2','TP3','TP4','TP5','TP6','TP7','TP8','TP9',}), xlim([L(1) L(end)])
            if ind == 12 && rp == 3
                eval(['h2=plot(L,V',int2str(i),'c,"-kx","MarkerSize",10);']);
                legend([h1,h2],{'Predicted','True'})
            end
        end
        for mp = 1:12
            figure(mp)
            switch rp
              case 1
                plot(nsp(ind),lsm(mp),'-sg','MarkerSize',10), hold on
              case 2
                plot(nsp(ind),lsm(mp),'-xb','MarkerSize',10), hold on            
              case 3
                plot(nsp(ind),lsm(mp),'-ok','MarkerSize',10), hold on            
            end
            % true values
            if ind == 1 && rp == 1
                switch mp
                  case 1
                    fill([nsp,fliplr(nsp)],[minfn,fliplr(maxfn)],'red','FaceAlpha',.3)         % Confidence int
                  case 4
                    fill([nsp,fliplr(nsp)],100*[minzeta,fliplr(maxzeta)],'red','FaceAlpha',.3) % Confidence int
                  otherwise
                    plot([nsp(1),nsp(end)],[tcp(mp),tcp(mp)],'r--')    
                end
                switch true
                  case mp<=3
                    xlabel('Pacing rate (Hz)')        
                    ylabel(['$f_',int2str(mp),'$ (Hz)'])
                  case mp<=6
                    xlabel('Pacing rate (Hz)')
                    ylabel(['$\zeta_',int2str(mp-3),'$ (\%)'])
                  case mp<=9
                    xlabel('Pacing rate (Hz)')
                    ylabel(['$m_',int2str(mp-6),'$ (kg)'])
                  case mp<=12
                    xlabel('Pacing rate (Hz)')
                    ylabel(['MAC ',int2str(mp-9)])
                end
            end
            if ind == 12 && rp == 3                
                vline(tcp(1)/2,'--k'), vline(tcp(1)/3,'-.k')
            end
        end % mp
    end    
end    
ferr=(mean(amp,3,'omitnan')-tcp)./tcp*100
[ferr(:,1:3),ferr(:,10:12);
 ferr(:,4:9)]